from __future__ import annotations

from typing import Dict, List, Iterable, Optional, cast

import httpx

from ..types import AgentOutput
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..types.chat import HeaderParam, ChatMessageParam
from ..types.agent import (
    Agent,
    AgentListParams,
    AgentCreateParams,
    AgentUpdateParams,
    AgentBulkDeleteParams,
    AgentDetectStatefulness,
    AgentTestConnectionParams,
    AgentDetectStatefulnessParams,
    AgentGenerateCompletionParams,
    AgentAutofillDescriptionParams,
)
from .._base_client import make_request_options
from ..types.common import APIResponse

__all__ = ["AgentsResource", "AsyncAgentsResource"]


class AgentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AgentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        project_id: str,
        supported_languages: SequenceNotStr[str],
        url: str,
        headers: Dict[str, str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        stateful: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Agent:
        """Create a new agent in the specified project.

        Parameters
        ----------
        name : str
            Name of the agent.
        project_id : str
            Project ID to create the agent in.
        supported_languages : SequenceNotStr[str]
            Supported languages for the agent.
        url : str
            URL endpoint of the agent.
        headers : Dict[str, str] | Omit
            HTTP headers to include when calling the agent.
        description : str | None | Omit
            Human-readable description of the agent.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Agent
            The newly created agent.
        """
        headers_api: list[HeaderParam] = []

        if headers is not omit:
            headers_api = [
                HeaderParam(name=header_name, value=header_value)
                for header_name, header_value in cast(Dict[str, str], headers).items()
            ]

        response = self._post(
            "/v2/agents",
            body=maybe_transform(
                {
                    "name": name,
                    "headers": headers_api,
                    "project_id": project_id,
                    "supported_languages": supported_languages,
                    "url": url,
                    "description": description,
                    "stateful": stateful,
                },
                AgentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Agent],
        )

        return self._unwrap(response)

    def retrieve(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Agent:
        """Retrieve an agent by its ID.

        Parameters
        ----------
        agent_id : str
            ID of the agent to retrieve.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Agent
            The requested agent.

        Raises
        ------
        ValueError
            If ``agent_id`` is empty.
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        response = self._get(
            f"/v2/agents/{agent_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Agent],
        )

        return self._unwrap(response)

    def update(
        self,
        agent_id: str,
        *,
        description: Optional[str] | Omit = omit,
        headers: Dict[str, str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        supported_languages: Optional[SequenceNotStr[str]] | Omit = omit,
        url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Agent:
        """Update an existing agent's configuration.

        Parameters
        ----------
        agent_id : str
            ID of the agent to update.
        description : str | None | Omit
            Updated description of the agent.
        headers : Dict[str, str] | Omit
            Updated HTTP headers for agent calls.
        name : str | None | Omit
            Updated name of the agent.
        supported_languages : SequenceNotStr[str] | None | Omit
            Updated supported languages.
        url : str | None | Omit
            Updated URL endpoint of the agent.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Agent
            The updated agent.

        Raises
        ------
        ValueError
            If ``agent_id`` is empty.
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")

        headers_api: list[HeaderParam] | Omit
        if headers is omit:
            headers_api = omit
        else:
            headers_api = [
                HeaderParam(name=header_name, value=header_value)
                for header_name, header_value in cast(Dict[str, str], headers).items()
            ]

        response = self._patch(
            f"/v2/agents/{agent_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "headers": headers_api,
                    "name": name,
                    "supported_languages": supported_languages,
                    "url": url,
                },
                AgentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Agent],
        )

        return self._unwrap(response)

    def list(
        self,
        *,
        project_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[Agent]:
        """List all agents, optionally filtered by project.

        Parameters
        ----------
        project_id : str | None | Omit
            Project ID to filter agents by.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list[Agent]
            List of agents matching the criteria.
        """
        response = self._get(
            "/v2/agents",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"project_id": project_id}, AgentListParams),
            ),
            cast_to=APIResponse[List[Agent]],
        )

        return self._unwrap(response)

    def delete(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete an agent by its ID.

        Parameters
        ----------
        agent_id : str
            ID of the agent to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Raises
        ------
        ValueError
            If ``agent_id`` is empty.
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        response = self._delete(
            f"/v2/agents/{agent_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def bulk_delete(
        self,
        *,
        agent_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete multiple agents at once.

        Parameters
        ----------
        agent_ids : SequenceNotStr[str]
            IDs of the agents to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.
        """
        response = self._delete(
            "/v2/agents",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"agent_ids": agent_ids}, AgentBulkDeleteParams),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def generate_completion(
        self,
        agent_id: str,
        *,
        messages: Iterable[ChatMessageParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AgentOutput:
        """Send a chat conversation to the agent and get a completion response.

        Parameters
        ----------
        agent_id : str
            ID of the agent to generate a completion from.
        messages : Iterable[ChatMessageParam]
            Conversation messages to send to the agent.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        AgentOutput
            The agent's completion response.

        Raises
        ------
        ValueError
            If ``agent_id`` is empty.
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        response = self._post(
            f"/v2/agents/{agent_id}/generate-completion",
            body=maybe_transform({"messages": messages}, AgentGenerateCompletionParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[AgentOutput],
        )

        return self._unwrap(response)

    def test_connection(
        self,
        *,
        url: str,
        headers: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AgentOutput:
        """Test the connection to an agent endpoint without persisting it.

        Parameters
        ----------
        url : str
            URL endpoint to test the connection to.
        headers : Dict[str, str] | Omit
            HTTP headers to include in the test request.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        AgentOutput
            The agent's test response.
        """
        response = self._post(
            "/v2/agents/test-connection",
            body=maybe_transform(
                {
                    "url": url,
                    "headers": headers,
                },
                AgentTestConnectionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[AgentOutput],
        )

        return self._unwrap(response)

    def generate_description(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Auto-generate a description for an agent by probing its capabilities.

        Discovers the agent's tone, features, goals, and constraints, then
        formats the findings into a human-readable description.

        Parameters
        ----------
        agent_id : str
            ID of the agent to generate a description for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        str
            The generated agent description.

        Raises
        ------
        ValueError
            If ``agent_id`` is empty.
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        response = self._post(
            f"/v2/agents/{agent_id}/autofill-description",
            body=maybe_transform({}, AgentAutofillDescriptionParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[str],
        )

        return self._unwrap(response)

    def detect_statefulness(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AgentDetectStatefulness:
        """Detect whether the agent is stateful by analyzing its behavior.

        Parameters
        ----------
        agent_id : str
            ID of the agent to detect statefulness for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        AgentDetectStatefulness
            The statefulness detection result.

        Raises
        ------
        ValueError
            If ``agent_id`` is empty.
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        response = self._post(
            f"/v2/agents/{agent_id}/detect-stateful",
            body=maybe_transform({}, AgentDetectStatefulnessParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[AgentDetectStatefulness],
        )

        return self._unwrap(response)


class AsyncAgentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AsyncAgentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        project_id: str,
        supported_languages: SequenceNotStr[str],
        url: str,
        headers: Dict[str, str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        stateful: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Agent:
        """Create a new agent in the specified project.

        Parameters
        ----------
        name : str
            Name of the agent.
        project_id : str
            Project ID to create the agent in.
        supported_languages : SequenceNotStr[str]
            Supported languages for the agent.
        url : str
            URL endpoint of the agent.
        headers : Dict[str, str] | Omit
            HTTP headers to include when calling the agent.
        description : str | None | Omit
            Human-readable description of the agent.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Agent
            The newly created agent.
        """
        headers_api: list[HeaderParam] = []

        if headers is not omit:
            headers_api = [
                HeaderParam(name=header_name, value=header_value)
                for header_name, header_value in cast(Dict[str, str], headers).items()
            ]

        response = await self._post(
            "/v2/agents",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "headers": headers_api,
                    "project_id": project_id,
                    "supported_languages": supported_languages,
                    "url": url,
                    "description": description,
                    "stateful": stateful,
                },
                AgentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Agent],
        )

        return self._unwrap(response)

    async def retrieve(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Agent:
        """Retrieve an agent by its ID.

        Parameters
        ----------
        agent_id : str
            ID of the agent to retrieve.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Agent
            The requested agent.

        Raises
        ------
        ValueError
            If ``agent_id`` is empty.
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        response = await self._get(
            f"/v2/agents/{agent_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Agent],
        )

        return self._unwrap(response)

    async def update(
        self,
        agent_id: str,
        *,
        description: Optional[str] | Omit = omit,
        headers: Dict[str, str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        supported_languages: Optional[SequenceNotStr[str]] | Omit = omit,
        url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Agent:
        """Update an existing agent's configuration.

        Parameters
        ----------
        agent_id : str
            ID of the agent to update.
        description : str | None | Omit
            Updated description of the agent.
        headers : Dict[str, str] | Omit
            Updated HTTP headers for agent calls.
        name : str | None | Omit
            Updated name of the agent.
        supported_languages : SequenceNotStr[str] | None | Omit
            Updated supported languages.
        url : str | None | Omit
            Updated URL endpoint of the agent.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Agent
            The updated agent.

        Raises
        ------
        ValueError
            If ``agent_id`` is empty.
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")

        headers_api: list[HeaderParam] | Omit
        if headers is omit:
            headers_api = omit
        else:
            headers_api = [
                HeaderParam(name=header_name, value=header_value)
                for header_name, header_value in cast(Dict[str, str], headers).items()
            ]

        response = await self._patch(
            f"/v2/agents/{agent_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "headers": headers_api,
                    "name": name,
                    "supported_languages": supported_languages,
                    "url": url,
                },
                AgentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Agent],
        )

        return self._unwrap(response)

    async def list(
        self,
        *,
        project_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[Agent]:
        """List all agents, optionally filtered by project.

        Parameters
        ----------
        project_id : str | None | Omit
            Project ID to filter agents by.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list[Agent]
            List of agents matching the criteria.
        """
        response = await self._get(
            "/v2/agents",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"project_id": project_id}, AgentListParams),
            ),
            cast_to=APIResponse[List[Agent]],
        )

        return self._unwrap(response)

    async def delete(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete an agent by its ID.

        Parameters
        ----------
        agent_id : str
            ID of the agent to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Raises
        ------
        ValueError
            If ``agent_id`` is empty.
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        response = await self._delete(
            f"/v2/agents/{agent_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def bulk_delete(
        self,
        *,
        agent_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete multiple agents at once.

        Parameters
        ----------
        agent_ids : SequenceNotStr[str]
            IDs of the agents to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.
        """
        response = await self._delete(
            "/v2/agents",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"agent_ids": agent_ids}, AgentBulkDeleteParams),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def generate_completion(
        self,
        agent_id: str,
        *,
        messages: Iterable[ChatMessageParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AgentOutput:
        """Send a chat conversation to the agent and get a completion response.

        Parameters
        ----------
        agent_id : str
            ID of the agent to generate a completion from.
        messages : Iterable[ChatMessageParam]
            Conversation messages to send to the agent.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        AgentOutput
            The agent's completion response.

        Raises
        ------
        ValueError
            If ``agent_id`` is empty.
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        response = await self._post(
            f"/v2/agents/{agent_id}/generate-completion",
            body=await async_maybe_transform({"messages": messages}, AgentGenerateCompletionParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[AgentOutput],
        )

        return self._unwrap(response)

    async def test_connection(
        self,
        *,
        url: str,
        headers: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AgentOutput:
        """Test the connection to an agent endpoint without persisting it.

        Parameters
        ----------
        url : str
            URL endpoint to test the connection to.
        headers : Dict[str, str] | Omit
            HTTP headers to include in the test request.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        AgentOutput
            The agent's test response.
        """
        response = await self._post(
            "/v2/agents/test-connection",
            body=await async_maybe_transform(
                {
                    "url": url,
                    "headers": headers,
                },
                AgentTestConnectionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[AgentOutput],
        )

        return self._unwrap(response)

    async def generate_description(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Auto-generate a description for an agent by probing its capabilities.

        Discovers the agent's tone, features, goals, and constraints, then
        formats the findings into a human-readable description.

        Parameters
        ----------
        agent_id : str
            ID of the agent to generate a description for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        str
            The generated agent description.

        Raises
        ------
        ValueError
            If ``agent_id`` is empty.
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        response = await self._post(
            f"/v2/agents/{agent_id}/autofill-description",
            body=await async_maybe_transform({}, AgentAutofillDescriptionParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[str],
        )

        return self._unwrap(response)

    async def detect_statefulness(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AgentDetectStatefulness:
        """Detect whether the agent is stateful by analyzing its behavior.

        Parameters
        ----------
        agent_id : str
            ID of the agent to detect statefulness for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        AgentDetectStatefulness
            The statefulness detection result.

        Raises
        ------
        ValueError
            If ``agent_id`` is empty.
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        response = await self._post(
            f"/v2/agents/{agent_id}/detect-stateful",
            body=await async_maybe_transform({}, AgentDetectStatefulnessParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[AgentDetectStatefulness],
        )

        return self._unwrap(response)


class AgentsResourceWithRawResponse:
    def __init__(self, agents: AgentsResource) -> None:
        self._agents = agents

        self.create = to_raw_response_wrapper(
            agents.create,
        )
        self.retrieve = to_raw_response_wrapper(
            agents.retrieve,
        )
        self.update = to_raw_response_wrapper(
            agents.update,
        )
        self.list = to_raw_response_wrapper(
            agents.list,
        )
        self.delete = to_raw_response_wrapper(
            agents.delete,
        )
        self.bulk_delete = to_raw_response_wrapper(
            agents.bulk_delete,
        )
        self.generate_completion = to_raw_response_wrapper(
            agents.generate_completion,
        )
        self.test_connection = to_raw_response_wrapper(
            agents.test_connection,
        )
        self.generate_description = to_raw_response_wrapper(
            agents.generate_description,
        )
        self.detect_statefulness = to_raw_response_wrapper(
            agents.detect_statefulness,
        )


class AsyncAgentsResourceWithRawResponse:
    def __init__(self, agents: AsyncAgentsResource) -> None:
        self._agents = agents

        self.create = async_to_raw_response_wrapper(
            agents.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            agents.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            agents.update,
        )
        self.list = async_to_raw_response_wrapper(
            agents.list,
        )
        self.delete = async_to_raw_response_wrapper(
            agents.delete,
        )
        self.bulk_delete = async_to_raw_response_wrapper(
            agents.bulk_delete,
        )
        self.generate_completion = async_to_raw_response_wrapper(
            agents.generate_completion,
        )
        self.test_connection = async_to_raw_response_wrapper(
            agents.test_connection,
        )
        self.generate_description = async_to_raw_response_wrapper(
            agents.generate_description,
        )
        self.detect_statefulness = async_to_raw_response_wrapper(
            agents.detect_statefulness,
        )


class AgentsResourceWithStreamingResponse:
    def __init__(self, agents: AgentsResource) -> None:
        self._agents = agents

        self.create = to_streamed_response_wrapper(
            agents.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            agents.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            agents.update,
        )
        self.list = to_streamed_response_wrapper(
            agents.list,
        )
        self.delete = to_streamed_response_wrapper(
            agents.delete,
        )
        self.bulk_delete = to_streamed_response_wrapper(
            agents.bulk_delete,
        )
        self.generate_completion = to_streamed_response_wrapper(
            agents.generate_completion,
        )
        self.test_connection = to_streamed_response_wrapper(
            agents.test_connection,
        )
        self.generate_description = to_streamed_response_wrapper(
            agents.generate_description,
        )
        self.detect_statefulness = to_streamed_response_wrapper(
            agents.detect_statefulness,
        )


class AsyncAgentsResourceWithStreamingResponse:
    def __init__(self, agents: AsyncAgentsResource) -> None:
        self._agents = agents

        self.create = async_to_streamed_response_wrapper(
            agents.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            agents.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            agents.update,
        )
        self.list = async_to_streamed_response_wrapper(
            agents.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            agents.delete,
        )
        self.bulk_delete = async_to_streamed_response_wrapper(
            agents.bulk_delete,
        )
        self.generate_completion = async_to_streamed_response_wrapper(
            agents.generate_completion,
        )
        self.test_connection = async_to_streamed_response_wrapper(
            agents.test_connection,
        )
        self.generate_description = async_to_streamed_response_wrapper(
            agents.generate_description,
        )
        self.detect_statefulness = async_to_streamed_response_wrapper(
            agents.detect_statefulness,
        )



"""Execution status and error types."""

from typing import Dict, Literal, Optional, TypedDict
from typing_extensions import Required

from .._models import BaseModel

__all__ = [
    "ExecutionError",
    "ExecutionErrorParam",
    "ErrorExecutionStatus",
    "ErrorExecutionStatusParam",
    "SuccessExecutionStatus",
    "SuccessExecutionStatusParam",
]


class ExecutionError(BaseModel):
    message: str
    details: Optional[Dict[str, object]] = None


class ExecutionErrorParam(TypedDict, total=False):
    message: Required[str]
    details: Dict[str, object]


class ErrorExecutionStatus(BaseModel):
    error_message: str
    status: Literal["error"]


class ErrorExecutionStatusParam(TypedDict, total=False):
    error_message: Required[str]
    status: Literal["error"]


class SuccessExecutionStatus(BaseModel):
    evaluation_id: str
    status: Literal["success"]


class SuccessExecutionStatusParam(TypedDict, total=False):
    evaluation_id: Required[str]
    status: Literal["success"]



"""Playground chat domain types."""

from typing import List, Literal, Optional, TypedDict
from datetime import datetime
from typing_extensions import Required

from .chat import ChatMessageWithMetadata
from .user import UserReference
from .agent import Agent, AgentReference
from .._types import SequenceNotStr
from .._models import BaseModel

__all__ = [
    "PlaygroundChat",
    "PlaygroundChatListParams",
    "PlaygroundChatRetrieveParams",
    "PlaygroundChatBulkDeleteParams",
]


# ---------------------------------------------------------------------------
# Models
# ---------------------------------------------------------------------------


class PlaygroundChat(BaseModel):
    id: str
    project_id: str
    created_at: datetime
    updated_at: datetime
    user: Optional[UserReference] = None
    agent: Optional[AgentReference | Agent] = None
    messages: List[ChatMessageWithMetadata]


# ---------------------------------------------------------------------------
# Params
# ---------------------------------------------------------------------------


class PlaygroundChatListParams(TypedDict, total=False):
    project_id: str
    include: Optional[List[Literal["agent"]]]
    limit: Optional[int]
    offset: Optional[int]


class PlaygroundChatRetrieveParams(TypedDict, total=False):
    include: Optional[List[Literal["agent"]]]


class PlaygroundChatBulkDeleteParams(TypedDict, total=False):
    chat_ids: Required[SequenceNotStr[str]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, List, Tuple, cast

import pytest

from giskard_hub import HubClient, AsyncHubClient
from tests.utils import assert_matches_type
from giskard_hub.types import Audit, AuditDisplay, APIPaginatedMetadata

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAuditLogs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search(self, client: HubClient) -> None:
        audit = client.audit_logs.search()
        assert_matches_type(List[Audit], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search_with_all_params(self, client: HubClient) -> None:
        audit = client.audit_logs.search(
            filters={"action": {"selected_options": ["insert"], "match_logic": "any"}},
            limit=10,
            offset=0,
        )
        assert_matches_type(List[Audit], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search_with_metadata(self, client: HubClient) -> None:
        audit = client.audit_logs.search(include_metadata=True)
        assert_matches_type(Tuple[List[Audit], APIPaginatedMetadata], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_search(self, client: HubClient) -> None:
        response = client.audit_logs.with_raw_response.search()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        audit = response.parse()
        assert_matches_type(List[Audit], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_search(self, client: HubClient) -> None:
        with client.audit_logs.with_streaming_response.search() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            audit = response.parse()
            assert_matches_type(List[Audit], audit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_entities(self, client: HubClient) -> None:
        audit = client.audit_logs.list_entities(
            entity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            entity_type="project",
        )
        assert_matches_type(List[AuditDisplay], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_entities_with_all_params(self, client: HubClient) -> None:
        audit = client.audit_logs.list_entities(
            entity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            entity_type="project",
            limit=10,
            offset=0,
        )
        assert_matches_type(List[AuditDisplay], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_entities_with_metadata(self, client: HubClient) -> None:
        audit = client.audit_logs.list_entities(
            entity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            entity_type="project",
            include_metadata=True,
        )
        assert_matches_type(Tuple[List[AuditDisplay], APIPaginatedMetadata], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_entities(self, client: HubClient) -> None:
        response = client.audit_logs.with_raw_response.list_entities(
            entity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            entity_type="project",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        audit = response.parse()
        assert_matches_type(List[AuditDisplay], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_entities(self, client: HubClient) -> None:
        with client.audit_logs.with_streaming_response.list_entities(
            entity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            entity_type="project",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            audit = response.parse()
            assert_matches_type(List[AuditDisplay], audit, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncAuditLogs:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search(self, async_client: AsyncHubClient) -> None:
        audit = await async_client.audit_logs.search()
        assert_matches_type(List[Audit], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search_with_all_params(self, async_client: AsyncHubClient) -> None:
        audit = await async_client.audit_logs.search(
            filters={"action": {"selected_options": ["insert"], "match_logic": "any"}},
            limit=10,
            offset=0,
        )
        assert_matches_type(List[Audit], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search_with_metadata(self, async_client: AsyncHubClient) -> None:
        audit = await async_client.audit_logs.search(include_metadata=True)
        assert_matches_type(Tuple[List[Audit], APIPaginatedMetadata], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_search(self, async_client: AsyncHubClient) -> None:
        response = await async_client.audit_logs.with_raw_response.search()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        audit = await response.parse()
        assert_matches_type(List[Audit], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_search(self, async_client: AsyncHubClient) -> None:
        async with async_client.audit_logs.with_streaming_response.search() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            audit = await response.parse()
            assert_matches_type(List[Audit], audit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_entities(self, async_client: AsyncHubClient) -> None:
        audit = await async_client.audit_logs.list_entities(
            entity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            entity_type="project",
        )
        assert_matches_type(List[AuditDisplay], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_entities_with_all_params(self, async_client: AsyncHubClient) -> None:
        audit = await async_client.audit_logs.list_entities(
            entity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            entity_type="project",
            limit=10,
            offset=0,
        )
        assert_matches_type(List[AuditDisplay], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_entities_with_metadata(self, async_client: AsyncHubClient) -> None:
        audit = await async_client.audit_logs.list_entities(
            entity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            entity_type="project",
            include_metadata=True,
        )
        assert_matches_type(Tuple[List[AuditDisplay], APIPaginatedMetadata], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_entities(self, async_client: AsyncHubClient) -> None:
        response = await async_client.audit_logs.with_raw_response.list_entities(
            entity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            entity_type="project",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        audit = await response.parse()
        assert_matches_type(List[AuditDisplay], audit, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_entities(self, async_client: AsyncHubClient) -> None:
        async with async_client.audit_logs.with_streaming_response.list_entities(
            entity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            entity_type="project",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            audit = await response.parse()
            assert_matches_type(List[AuditDisplay], audit, path=["response"])

        assert cast(Any, response.is_closed) is True



from .projects import (
    ProjectsResource,
    AsyncProjectsResource,
    ProjectsResourceWithRawResponse,
    AsyncProjectsResourceWithRawResponse,
    ProjectsResourceWithStreamingResponse,
    AsyncProjectsResourceWithStreamingResponse,
)
from .scenarios import (
    ScenariosResource,
    AsyncScenariosResource,
    ScenariosResourceWithRawResponse,
    AsyncScenariosResourceWithRawResponse,
    ScenariosResourceWithStreamingResponse,
    AsyncScenariosResourceWithStreamingResponse,
)

__all__ = [
    "ProjectsResource",
    "AsyncProjectsResource",
    "ProjectsResourceWithRawResponse",
    "AsyncProjectsResourceWithRawResponse",
    "ProjectsResourceWithStreamingResponse",
    "AsyncProjectsResourceWithStreamingResponse",
    "ScenariosResource",
    "AsyncScenariosResource",
    "ScenariosResourceWithRawResponse",
    "AsyncScenariosResourceWithRawResponse",
    "ScenariosResourceWithStreamingResponse",
    "AsyncScenariosResourceWithStreamingResponse",
]



"""User types."""

from typing import Optional

from .._models import BaseModel

__all__ = ["User", "UserReference"]


class UserReference(BaseModel):
    id: str
    name: str


class User(BaseModel):
    id: str
    email: str
    name: Optional[str] = None



from __future__ import annotations

from typing import List

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.scan import ScanProbe, ScanProbeAttempt
from ..._base_client import make_request_options
from ...types.common import APIResponse

__all__ = ["ProbesResource", "AsyncProbesResource"]


class ProbesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ProbesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return ProbesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProbesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return ProbesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        probe_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScanProbe:
        """Retrieve a specific scan probe result by its ID.

        Parameters
        ----------
        probe_id : str
            Scan Probe Result ID to retrieve.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        ScanProbe
            The requested scan probe object.

        Raises
        ------
        ValueError
            If ``probe_id`` is empty.
        """
        if not probe_id:
            raise ValueError(f"Expected a non-empty value for `probe_id` but received {probe_id!r}")
        response = self._get(
            f"/v2/scan-probes/{probe_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[ScanProbe],
        )

        return self._unwrap(response)

    def list_attempts(
        self,
        probe_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[ScanProbeAttempt]:
        """List all attempts for a specific scan probe.

        Parameters
        ----------
        probe_id : str
            Scan Probe Result ID to list attempts.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list of ScanProbeAttempt
            A list of scan probe attempts.

        Raises
        ------
        ValueError
            If ``probe_id`` is empty.
        """
        if not probe_id:
            raise ValueError(f"Expected a non-empty value for `probe_id` but received {probe_id!r}")
        response = self._get(
            f"/v2/scan-probes/{probe_id}/attempts",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[List[ScanProbeAttempt]],
        )

        return self._unwrap(response)


class AsyncProbesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncProbesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AsyncProbesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProbesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AsyncProbesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        probe_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScanProbe:
        """Retrieve a specific scan probe result by its ID.

        Parameters
        ----------
        probe_id : str
            Scan Probe Result ID to retrieve.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        ScanProbe
            The requested scan probe object.

        Raises
        ------
        ValueError
            If ``probe_id`` is empty.
        """
        if not probe_id:
            raise ValueError(f"Expected a non-empty value for `probe_id` but received {probe_id!r}")
        response = await self._get(
            f"/v2/scan-probes/{probe_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[ScanProbe],
        )

        return self._unwrap(response)

    async def list_attempts(
        self,
        probe_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[ScanProbeAttempt]:
        """List all attempts for a specific scan probe.

        Parameters
        ----------
        probe_id : str
            Scan Probe Result ID to list attempts.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list of ScanProbeAttempt
            A list of scan probe attempts.

        Raises
        ------
        ValueError
            If ``probe_id`` is empty.
        """
        if not probe_id:
            raise ValueError(f"Expected a non-empty value for `probe_id` but received {probe_id!r}")
        response = await self._get(
            f"/v2/scan-probes/{probe_id}/attempts",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[List[ScanProbeAttempt]],
        )

        return self._unwrap(response)


class ProbesResourceWithRawResponse:
    def __init__(self, probes: ProbesResource) -> None:
        self._probes = probes

        self.retrieve = to_raw_response_wrapper(
            probes.retrieve,
        )
        self.list_attempts = to_raw_response_wrapper(
            probes.list_attempts,
        )


class AsyncProbesResourceWithRawResponse:
    def __init__(self, probes: AsyncProbesResource) -> None:
        self._probes = probes

        self.retrieve = async_to_raw_response_wrapper(
            probes.retrieve,
        )
        self.list_attempts = async_to_raw_response_wrapper(
            probes.list_attempts,
        )


class ProbesResourceWithStreamingResponse:
    def __init__(self, probes: ProbesResource) -> None:
        self._probes = probes

        self.retrieve = to_streamed_response_wrapper(
            probes.retrieve,
        )
        self.list_attempts = to_streamed_response_wrapper(
            probes.list_attempts,
        )


class AsyncProbesResourceWithStreamingResponse:
    def __init__(self, probes: AsyncProbesResource) -> None:
        self._probes = probes

        self.retrieve = async_to_streamed_response_wrapper(
            probes.retrieve,
        )
        self.list_attempts = async_to_streamed_response_wrapper(
            probes.list_attempts,
        )



"""Protocols, type aliases, and utility functions for the helpers resource."""

from __future__ import annotations

from typing import TYPE_CHECKING, Any, TypeVar, Protocol, runtime_checkable

from pydantic import TypeAdapter

from .._models import BaseModel
from .._resource import SyncAPIResource, AsyncAPIResource
from ..types.chat import ChatMessage
from ..types.scan import Scan, ScanProbe
from ..types.agent import Agent, AgentOutput
from ..types.common import TaskState
from ..types.dataset import Dataset
from ..types.evaluation import Evaluation, TestCaseEvaluation
from ..types.knowledge_base import KnowledgeBase

if TYPE_CHECKING:
    from .._client import HubClient, AsyncHubClient

__all__ = [
    "StatefulEntity",
    "RetrievableResource",
    "AsyncRetrievableResource",
    "TStateful",
    "AgentReturn",
    "agent_return_adapter",
    "PrintMetricsEntity",
    "normalize_agent_output",
    "map_entity_to_resource",
]


# ---------------------------------------------------------------------------
# Protocols
# ---------------------------------------------------------------------------


@runtime_checkable
class StatefulEntity(Protocol):
    """Protocol for entities that expose task-like fields."""

    id: str

    @property
    def state(self) -> TaskState: ...


class RetrievableResource(Protocol):
    def retrieve(self, id: str) -> StatefulEntity: ...


class AsyncRetrievableResource(Protocol):
    async def retrieve(self, id: str) -> StatefulEntity: ...


# ---------------------------------------------------------------------------
# Type aliases & adapters
# ---------------------------------------------------------------------------

TStateful = TypeVar("TStateful", bound=StatefulEntity)

AgentReturn = str | ChatMessage | AgentOutput
agent_return_adapter: TypeAdapter[AgentReturn] = TypeAdapter(AgentReturn)

PrintMetricsEntity = Evaluation | Scan


# ---------------------------------------------------------------------------
# Utility functions
# ---------------------------------------------------------------------------


def normalize_agent_output(value: Any) -> AgentOutput:
    """Validate and normalize arbitrary agent return values into an ``AgentOutput``."""
    parsed: AgentReturn = agent_return_adapter.validate_python(value)

    match parsed:
        case AgentOutput():
            return parsed
        case ChatMessage():
            return AgentOutput(response=parsed)
        case str():
            return AgentOutput(response=ChatMessage(role="assistant", content=parsed))
        case _:
            raise ValueError(f"Invalid agent output: {value!r}")


def map_entity_to_resource(
    client: "HubClient | AsyncHubClient",
    entity: BaseModel,
) -> SyncAPIResource | AsyncAPIResource:
    """Map a model instance to the corresponding API resource on a client.

    Parameters
    ----------
    client :
        The API client that exposes resource attributes such as ``agents``,
        ``datasets``, ``evaluations``, etc.
    entity :
        The entity instance whose managing resource should be resolved.

    Returns
    -------
    SyncAPIResource or AsyncAPIResource
        The API resource responsible for the given entity type.

    Raises
    ------
    TypeError
        If the entity type is not supported.
    """
    if isinstance(entity, Agent):
        return client.agents
    if isinstance(entity, Dataset):
        return client.datasets
    if isinstance(entity, Evaluation):
        return client.evaluations
    if isinstance(entity, KnowledgeBase):
        return client.knowledge_bases
    if isinstance(entity, Scan):
        return client.scans
    if isinstance(entity, ScanProbe):
        return client.scans.probes
    if isinstance(entity, TestCaseEvaluation):
        return client.evaluations.results
    raise TypeError(f"Unsupported entity type for wait_for_completion: {type(entity)!r}")



from __future__ import annotations

import httpx

from ...types import TestCaseComment
from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.common import APIResponse
from ...types.test_case import CommentAddParams, CommentEditParams

__all__ = ["CommentsResource", "AsyncCommentsResource"]


class CommentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CommentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return CommentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CommentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return CommentsResourceWithStreamingResponse(self)

    def delete(
        self,
        comment_id: str,
        *,
        test_case_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a comment from a test case.

        Parameters
        ----------
        comment_id : str
            Comment ID to delete.
        test_case_id : str
            Test Case ID to delete the comment from.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``test_case_id`` or ``comment_id`` is empty.
        """
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        if not comment_id:
            raise ValueError(f"Expected a non-empty value for `comment_id` but received {comment_id!r}")
        response = self._delete(
            f"/v2/test-cases/{test_case_id}/comments/{comment_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def add(
        self,
        test_case_id: str,
        *,
        content: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCaseComment:
        """
        Add a new comment to a test case.

        Parameters
        ----------
        test_case_id : str
            Test Case ID to add the comment to.
        content : str
            Content of the comment to add.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCaseComment
            The newly created comment.

        Raises
        ------
        ValueError
            If ``test_case_id`` is empty.
        """
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        response = self._post(
            f"/v2/test-cases/{test_case_id}/comments",
            body=maybe_transform({"comment": content}, CommentAddParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCaseComment],
        )

        return self._unwrap(response)

    def edit(
        self,
        comment_id: str,
        *,
        test_case_id: str,
        content: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCaseComment:
        """
        Edit an existing comment on a test case.

        Parameters
        ----------
        comment_id : str
            Comment ID to edit.
        test_case_id : str
            Test Case ID to edit the comment from.
        content : str
            Content of the comment to edit.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCaseComment
            The edited comment.

        Raises
        ------
        ValueError
            If ``test_case_id`` or ``comment_id`` is empty.
        """
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        if not comment_id:
            raise ValueError(f"Expected a non-empty value for `comment_id` but received {comment_id!r}")
        response = self._patch(
            f"/v2/test-cases/{test_case_id}/comments/{comment_id}",
            body=maybe_transform({"comment": content}, CommentEditParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCaseComment],
        )

        return self._unwrap(response)


class AsyncCommentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCommentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCommentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCommentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AsyncCommentsResourceWithStreamingResponse(self)

    async def delete(
        self,
        comment_id: str,
        *,
        test_case_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a comment from a test case.

        Parameters
        ----------
        comment_id : str
            Comment ID to delete.
        test_case_id : str
            Test Case ID to delete the comment from.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``test_case_id`` or ``comment_id`` is empty.
        """
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        if not comment_id:
            raise ValueError(f"Expected a non-empty value for `comment_id` but received {comment_id!r}")
        response = await self._delete(
            f"/v2/test-cases/{test_case_id}/comments/{comment_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def add(
        self,
        test_case_id: str,
        *,
        content: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCaseComment:
        """
        Add a new comment to a test case.

        Parameters
        ----------
        test_case_id : str
            Test Case ID to add the comment to.
        content : str
            Content of the comment to add.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCaseComment
            The newly created comment.

        Raises
        ------
        ValueError
            If ``test_case_id`` is empty.
        """
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        response = await self._post(
            f"/v2/test-cases/{test_case_id}/comments",
            body=await async_maybe_transform({"comment": content}, CommentAddParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCaseComment],
        )

        return self._unwrap(response)

    async def edit(
        self,
        comment_id: str,
        *,
        test_case_id: str,
        content: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCaseComment:
        """
        Edit an existing comment on a test case.

        Parameters
        ----------
        comment_id : str
            Comment ID to edit.
        test_case_id : str
            Test Case ID to edit the comment from.
        content : str
            Content of the comment to edit.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCaseComment
            The edited comment.

        Raises
        ------
        ValueError
            If ``test_case_id`` or ``comment_id`` is empty.
        """
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        if not comment_id:
            raise ValueError(f"Expected a non-empty value for `comment_id` but received {comment_id!r}")
        response = await self._patch(
            f"/v2/test-cases/{test_case_id}/comments/{comment_id}",
            body=await async_maybe_transform({"comment": content}, CommentEditParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCaseComment],
        )

        return self._unwrap(response)


class CommentsResourceWithRawResponse:
    def __init__(self, comments: CommentsResource) -> None:
        self._comments = comments

        self.delete = to_raw_response_wrapper(
            comments.delete,
        )
        self.add = to_raw_response_wrapper(
            comments.add,
        )
        self.edit = to_raw_response_wrapper(
            comments.edit,
        )


class AsyncCommentsResourceWithRawResponse:
    def __init__(self, comments: AsyncCommentsResource) -> None:
        self._comments = comments

        self.delete = async_to_raw_response_wrapper(
            comments.delete,
        )
        self.add = async_to_raw_response_wrapper(
            comments.add,
        )
        self.edit = async_to_raw_response_wrapper(
            comments.edit,
        )


class CommentsResourceWithStreamingResponse:
    def __init__(self, comments: CommentsResource) -> None:
        self._comments = comments

        self.delete = to_streamed_response_wrapper(
            comments.delete,
        )
        self.add = to_streamed_response_wrapper(
            comments.add,
        )
        self.edit = to_streamed_response_wrapper(
            comments.edit,
        )


class AsyncCommentsResourceWithStreamingResponse:
    def __init__(self, comments: AsyncCommentsResource) -> None:
        self._comments = comments

        self.delete = async_to_streamed_response_wrapper(
            comments.delete,
        )
        self.add = async_to_streamed_response_wrapper(
            comments.add,
        )
        self.edit = async_to_streamed_response_wrapper(
            comments.edit,
        )



from __future__ import annotations

from typing import List, Literal, Optional

import httpx

from .probes import (
    ProbesResource,
    AsyncProbesResource,
    ProbesResourceWithRawResponse,
    AsyncProbesResourceWithRawResponse,
    ProbesResourceWithStreamingResponse,
    AsyncProbesResourceWithStreamingResponse,
)
from ...types import (
    Scan,
    Agent,
    ScanProbe,
    ScanCategory,
    KnowledgeBase,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from .attempts import (
    AttemptsResource,
    AsyncAttemptsResource,
    AttemptsResourceWithRawResponse,
    AsyncAttemptsResourceWithRawResponse,
    AttemptsResourceWithStreamingResponse,
    AsyncAttemptsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from .._included import embed_included_list, embed_included_single
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.scan import (
    ScanListParams,
    ScanCreateParams,
    ScanRetrieveParams,
    ScanBulkDeleteParams,
)
from ..._base_client import make_request_options
from ...types.common import APIResponse, APIResponseWithIncluded

__all__ = ["ScansResource", "AsyncScansResource"]


class ScansResource(SyncAPIResource):
    @cached_property
    def probes(self) -> ProbesResource:
        return ProbesResource(self._client)

    @cached_property
    def attempts(self) -> AttemptsResource:
        return AttemptsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ScansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return ScansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ScansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return ScansResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        project_id: str,
        agent_id: str,
        knowledge_base_id: Optional[str] | Omit = omit,
        probe_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Scan:
        """Launch a new vulnerability scan of an agent.

        Parameters
        ----------
        project_id : str
            Project ID to use for the scan.
        agent_id : str
            Agent ID to use for the scan.
        knowledge_base_id : str or None
            Knowledge Base ID to use for the scan.
        probe_ids : list of str or None
            List of specific probe IDs to run in the scan.
        tags : list of str or None
            List of category tags to apply to the scan; use ``list_categories``
            to get the available categories.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Scan
            The newly created scan object.
        """
        response = self._post(
            "/v2/scans",
            body=maybe_transform(
                {
                    "agent_id": agent_id,
                    "project_id": project_id,
                    "knowledge_base_id": knowledge_base_id,
                    "probe_ids": probe_ids,
                    "tags": tags,
                },
                ScanCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Scan],
        )

        return self._unwrap(response)

    def retrieve(
        self,
        scan_id: str,
        *,
        include: Optional[List[Literal["agent", "knowledge_base"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Scan:
        """Retrieve a scan result by its ID, with optional related resource inclusion.

        Parameters
        ----------
        scan_id : str
            Scan Result ID.
        include : list of {'agent', 'knowledge_base'} or None
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Scan
            The requested scan object.

        Raises
        ------
        ValueError
            If ``scan_id`` is empty.
        """
        if not scan_id:
            raise ValueError(f"Expected a non-empty value for `scan_id` but received {scan_id!r}")
        response = self._get(
            f"/v2/scans/{scan_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"include": include}, ScanRetrieveParams),
            ),
            cast_to=APIResponseWithIncluded[Scan, APIResponse[Agent | KnowledgeBase]],
        )

        if include is not omit and include:
            response = embed_included_single(response, id_getter=lambda scan_result: scan_result.id)

        return self._unwrap(response)

    def list(
        self,
        *,
        project_id: Optional[str] | Omit = omit,
        include: Optional[List[Literal["agent", "knowledge_base"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[Scan]:
        """List all scan results, optionally filtered by project.

        Parameters
        ----------
        project_id : str or None
            Project ID to use for the scan.
        include : list of {'agent', 'knowledge_base'} or None
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list of Scan
            A list of scan objects.
        """
        response = self._get(
            "/v2/scans",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include": include,
                        "project_id": project_id,
                    },
                    ScanListParams,
                ),
            ),
            cast_to=APIResponseWithIncluded[List[Scan], APIResponse[Agent | KnowledgeBase]],
        )

        if include is not omit and include:
            response = embed_included_list(response, id_getter=lambda scan_result: scan_result.id)

        return self._unwrap(response)

    def delete(
        self,
        scan_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a scan result by its ID.

        Parameters
        ----------
        scan_id : str
            Scan Result ID to delete.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``scan_id`` is empty.
        """
        if not scan_id:
            raise ValueError(f"Expected a non-empty value for `scan_id` but received {scan_id!r}")
        response = self._delete(
            f"/v2/scans/{scan_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def bulk_delete(
        self,
        *,
        scan_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete multiple scan results at once.

        Parameters
        ----------
        scan_ids : list of str
            List of scan IDs to delete.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None
        """
        response = self._delete(
            "/v2/scans",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"scan_ids": scan_ids}, ScanBulkDeleteParams),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def list_categories(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[ScanCategory]:
        """List all available scan vulnerability categories.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list of ScanCategory
            A list of scan vulnerability categories.
        """
        response = self._get(
            "/v2/scan-categories",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[List[ScanCategory]],
        )

        return self._unwrap(response)

    def list_probes(
        self,
        scan_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[ScanProbe]:
        """List all probes for a given scan result.

        Parameters
        ----------
        scan_id : str
            Scan Result ID to list probes.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list of ScanProbe
            A list of scan probes for the given scan.

        Raises
        ------
        ValueError
            If ``scan_id`` is empty.
        """
        if not scan_id:
            raise ValueError(f"Expected a non-empty value for `scan_id` but received {scan_id!r}")
        response = self._get(
            f"/v2/scans/{scan_id}/probes",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[List[ScanProbe]],
        )

        return self._unwrap(response)


class AsyncScansResource(AsyncAPIResource):
    @cached_property
    def probes(self) -> AsyncProbesResource:
        return AsyncProbesResource(self._client)

    @cached_property
    def attempts(self) -> AsyncAttemptsResource:
        return AsyncAttemptsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncScansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AsyncScansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncScansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AsyncScansResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        project_id: str,
        agent_id: str,
        knowledge_base_id: Optional[str] | Omit = omit,
        probe_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Scan:
        """Launch a new vulnerability scan of an agent.

        Parameters
        ----------
        project_id : str
            Project ID to use for the scan.
        agent_id : str
            Agent ID to use for the scan.
        knowledge_base_id : str or None
            Knowledge Base ID to use for the scan.
        probe_ids : list of str or None
            List of specific probe IDs to run in the scan.
        tags : list of str or None
            List of category tags to apply to the scan; use ``list_categories``
            to get the available categories.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Scan
            The newly created scan object.
        """
        response = await self._post(
            "/v2/scans",
            body=await async_maybe_transform(
                {
                    "agent_id": agent_id,
                    "project_id": project_id,
                    "knowledge_base_id": knowledge_base_id,
                    "probe_ids": probe_ids,
                    "tags": tags,
                },
                ScanCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Scan],
        )

        return self._unwrap(response)

    async def retrieve(
        self,
        scan_id: str,
        *,
        include: Optional[List[Literal["agent", "knowledge_base"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Scan:
        """Retrieve a scan result by its ID, with optional related resource inclusion.

        Parameters
        ----------
        scan_id : str
            Scan Result ID to retrieve.
        include : list of {'agent', 'knowledge_base'} or None
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Scan
            The requested scan object.

        Raises
        ------
        ValueError
            If ``scan_id`` is empty.
        """
        if not scan_id:
            raise ValueError(f"Expected a non-empty value for `scan_id` but received {scan_id!r}")

        response = await self._get(
            f"/v2/scans/{scan_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"include": include}, ScanRetrieveParams),
            ),
            cast_to=APIResponseWithIncluded[Scan, APIResponse[Agent | KnowledgeBase]],
        )

        if include is not omit and include:
            response = embed_included_single(response, id_getter=lambda scan_result: scan_result.id)

        return self._unwrap(response)

    async def list(
        self,
        *,
        project_id: Optional[str] | Omit = omit,
        include: Optional[List[Literal["agent", "knowledge_base"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[Scan]:
        """List all scan results, optionally filtered by project.

        Parameters
        ----------
        project_id : str or None
            Project ID to use for the scan.
        include : list of {'agent', 'knowledge_base'} or None
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list of Scan
            A list of scan objects.
        """
        response = await self._get(
            "/v2/scans",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "include": include,
                        "project_id": project_id,
                    },
                    ScanListParams,
                ),
            ),
            cast_to=APIResponseWithIncluded[List[Scan], APIResponse[Agent | KnowledgeBase]],
        )

        if include is not omit and include:
            response = embed_included_list(response, id_getter=lambda scan_result: scan_result.id)

        return self._unwrap(response)

    async def delete(
        self,
        scan_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a scan result by its ID.

        Parameters
        ----------
        scan_id : str
            Scan Result ID to delete.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``scan_id`` is empty.
        """
        if not scan_id:
            raise ValueError(f"Expected a non-empty value for `scan_id` but received {scan_id!r}")
        response = await self._delete(
            f"/v2/scans/{scan_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def bulk_delete(
        self,
        *,
        scan_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete multiple scan results at once.

        Parameters
        ----------
        scan_ids : list of str
            List of scan IDs to delete.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None
        """
        response = await self._delete(
            "/v2/scans",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"scan_ids": scan_ids}, ScanBulkDeleteParams),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def list_categories(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[ScanCategory]:
        """List all available scan vulnerability categories.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list of ScanCategory
            A list of scan vulnerability categories.
        """
        response = await self._get(
            "/v2/scan-categories",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[List[ScanCategory]],
        )

        return self._unwrap(response)

    async def list_probes(
        self,
        scan_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[ScanProbe]:
        """List all probes for a given scan result.

        Parameters
        ----------
        scan_id : str
            Scan Result ID to list probes.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list of ScanProbe
            A list of scan probes for the given scan.

        Raises
        ------
        ValueError
            If ``scan_id`` is empty.
        """
        if not scan_id:
            raise ValueError(f"Expected a non-empty value for `scan_id` but received {scan_id!r}")
        response = await self._get(
            f"/v2/scans/{scan_id}/probes",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[List[ScanProbe]],
        )

        return self._unwrap(response)


class ScansResourceWithRawResponse:
    def __init__(self, scans: ScansResource) -> None:
        self._scans = scans

        self.create = to_raw_response_wrapper(
            scans.create,
        )
        self.retrieve = to_raw_response_wrapper(
            scans.retrieve,
        )
        self.list = to_raw_response_wrapper(
            scans.list,
        )
        self.delete = to_raw_response_wrapper(
            scans.delete,
        )
        self.bulk_delete = to_raw_response_wrapper(
            scans.bulk_delete,
        )
        self.list_categories = to_raw_response_wrapper(
            scans.list_categories,
        )
        self.list_probes = to_raw_response_wrapper(
            scans.list_probes,
        )

    @cached_property
    def probes(self) -> ProbesResourceWithRawResponse:
        return ProbesResourceWithRawResponse(self._scans.probes)

    @cached_property
    def attempts(self) -> AttemptsResourceWithRawResponse:
        return AttemptsResourceWithRawResponse(self._scans.attempts)


class AsyncScansResourceWithRawResponse:
    def __init__(self, scans: AsyncScansResource) -> None:
        self._scans = scans

        self.create = async_to_raw_response_wrapper(
            scans.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            scans.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            scans.list,
        )
        self.delete = async_to_raw_response_wrapper(
            scans.delete,
        )
        self.bulk_delete = async_to_raw_response_wrapper(
            scans.bulk_delete,
        )
        self.list_categories = async_to_raw_response_wrapper(
            scans.list_categories,
        )
        self.list_probes = async_to_raw_response_wrapper(
            scans.list_probes,
        )

    @cached_property
    def probes(self) -> AsyncProbesResourceWithRawResponse:
        return AsyncProbesResourceWithRawResponse(self._scans.probes)

    @cached_property
    def attempts(self) -> AsyncAttemptsResourceWithRawResponse:
        return AsyncAttemptsResourceWithRawResponse(self._scans.attempts)


class ScansResourceWithStreamingResponse:
    def __init__(self, scans: ScansResource) -> None:
        self._scans = scans

        self.create = to_streamed_response_wrapper(
            scans.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            scans.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            scans.list,
        )
        self.delete = to_streamed_response_wrapper(
            scans.delete,
        )
        self.bulk_delete = to_streamed_response_wrapper(
            scans.bulk_delete,
        )
        self.list_categories = to_streamed_response_wrapper(
            scans.list_categories,
        )
        self.list_probes = to_streamed_response_wrapper(
            scans.list_probes,
        )

    @cached_property
    def probes(self) -> ProbesResourceWithStreamingResponse:
        return ProbesResourceWithStreamingResponse(self._scans.probes)

    @cached_property
    def attempts(self) -> AttemptsResourceWithStreamingResponse:
        return AttemptsResourceWithStreamingResponse(self._scans.attempts)


class AsyncScansResourceWithStreamingResponse:
    def __init__(self, scans: AsyncScansResource) -> None:
        self._scans = scans

        self.create = async_to_streamed_response_wrapper(
            scans.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            scans.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            scans.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            scans.delete,
        )
        self.bulk_delete = async_to_streamed_response_wrapper(
            scans.bulk_delete,
        )
        self.list_categories = async_to_streamed_response_wrapper(
            scans.list_categories,
        )
        self.list_probes = async_to_streamed_response_wrapper(
            scans.list_probes,
        )

    @cached_property
    def probes(self) -> AsyncProbesResourceWithStreamingResponse:
        return AsyncProbesResourceWithStreamingResponse(self._scans.probes)

    @cached_property
    def attempts(self) -> AsyncAttemptsResourceWithStreamingResponse:
        return AsyncAttemptsResourceWithStreamingResponse(self._scans.attempts)



from .scans import (
    ScansResource,
    AsyncScansResource,
    ScansResourceWithRawResponse,
    AsyncScansResourceWithRawResponse,
    ScansResourceWithStreamingResponse,
    AsyncScansResourceWithStreamingResponse,
)
from .probes import (
    ProbesResource,
    AsyncProbesResource,
    ProbesResourceWithRawResponse,
    AsyncProbesResourceWithRawResponse,
    ProbesResourceWithStreamingResponse,
    AsyncProbesResourceWithStreamingResponse,
)
from .attempts import (
    AttemptsResource,
    AsyncAttemptsResource,
    AttemptsResourceWithRawResponse,
    AsyncAttemptsResourceWithRawResponse,
    AttemptsResourceWithStreamingResponse,
    AsyncAttemptsResourceWithStreamingResponse,
)

__all__ = [
    "ProbesResource",
    "AsyncProbesResource",
    "ProbesResourceWithRawResponse",
    "AsyncProbesResourceWithRawResponse",
    "ProbesResourceWithStreamingResponse",
    "AsyncProbesResourceWithStreamingResponse",
    "AttemptsResource",
    "AsyncAttemptsResource",
    "AttemptsResourceWithRawResponse",
    "AsyncAttemptsResourceWithRawResponse",
    "AttemptsResourceWithStreamingResponse",
    "AsyncAttemptsResourceWithStreamingResponse",
    "ScansResource",
    "AsyncScansResource",
    "ScansResourceWithRawResponse",
    "AsyncScansResourceWithRawResponse",
    "ScansResourceWithStreamingResponse",
    "AsyncScansResourceWithStreamingResponse",
]



"""Audit domain types."""

from typing import Any, Dict, List, Literal, Optional, TypeAlias, TypedDict
from datetime import datetime

from .common import OrderByParam, FilterValueParam
from .._models import BaseModel

__all__ = [
    "ActionType",
    "AuditDiffKind",
    "AuditDiffItem",
    "Audit",
    "AuditDisplay",
    "AuditDisplayDiffItem",
    "AuditListEntityParams",
    "AuditSearchParams",
    "AuditOrderByParam",
    "AuditFiltersParam",
]


# ---------------------------------------------------------------------------
# Enums
# ---------------------------------------------------------------------------

ActionType: TypeAlias = Literal["insert", "update", "delete"]
AuditDiffKind: TypeAlias = Literal["added", "removed", "changed"]


# ---------------------------------------------------------------------------
# Models
# ---------------------------------------------------------------------------


class AuditDiffItem(BaseModel):
    kind: AuditDiffKind
    field: str
    old_value: Optional[Any] = None
    new_value: Optional[Any] = None


class Audit(BaseModel):
    id: str
    action: ActionType
    entity_id: str
    entity_type: str
    created_at: datetime
    user_id: Optional[str] = None
    project_id: Optional[str] = None
    diff: Optional[List[AuditDiffItem]] = None
    metadata: Optional[Dict[str, object]] = None


class AuditDisplayDiffItem(BaseModel):
    kind: Literal["added", "removed", "changed", "skip"]
    scope: str
    root: str
    skip_count: int | None = None
    label: str | None = None
    before_str: str | None = None
    after_str: str | None = None


class AuditDisplay(BaseModel):
    id: str
    created_at: datetime
    user_id: str
    user_name: Optional[str] = None
    action: ActionType
    diffs: List[AuditDisplayDiffItem]
    real_change_count: int
    summary_fields: List[str]


# ---------------------------------------------------------------------------
# Params
# ---------------------------------------------------------------------------

AuditSortColumn = Literal["action", "created_at", "entity_type", "project_id", "user_id"]
AuditFilterColumn = Literal["action", "created_at", "entity_type", "project_id", "user_id"]

AuditOrderByParam = OrderByParam[AuditSortColumn]
AuditFiltersParam = Dict[AuditFilterColumn, FilterValueParam]


class AuditListEntityParams(TypedDict, total=False):
    limit: Optional[int]
    offset: Optional[int]


class AuditSearchParams(TypedDict, total=False):
    search: Optional[str]
    order_by: Optional[List[AuditOrderByParam]]
    filters: Optional[AuditFiltersParam]
    limit: int
    offset: int



from __future__ import annotations

from typing import List, Literal, Optional

import httpx

from ..types import (
    Agent,
    PlaygroundChatListParams,
    PlaygroundChatRetrieveParams,
    PlaygroundChatBulkDeleteParams,
)
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from ._included import embed_included_list, embed_included_single
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.common import APIResponse, APIResponseWithIncluded
from ..types.playground_chat import PlaygroundChat

__all__ = ["PlaygroundChatsResource", "AsyncPlaygroundChatsResource"]


class PlaygroundChatsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PlaygroundChatsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return PlaygroundChatsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PlaygroundChatsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return PlaygroundChatsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        project_id: str,
        include: Optional[List[Literal["agent"]]] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        offset: Optional[int] | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[PlaygroundChat]:
        """List playground chats for a project with optional pagination.

        Parameters
        ----------
        project_id : str
            Project ID to list playground chats for.
        include : list[Literal["agent"]] | None | Omit
            Related resources to embed in the response (e.g. ``["agent"]``).
        limit : int | None | Omit
            Maximum number of playground chats to return.
        offset : int | None | Omit
            Number of results to skip for pagination.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list[PlaygroundChat]
            List of playground chats for the project.
        """
        response = self._get(
            "/v2/playground-chats",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "project_id": project_id,
                        "include": include,
                        "limit": limit,
                        "offset": offset,
                    },
                    PlaygroundChatListParams,
                ),
            ),
            cast_to=APIResponseWithIncluded[List[PlaygroundChat], APIResponse[Agent]],
        )

        if include is not omit and include:
            response = embed_included_list(response, id_getter=lambda playground_chat: playground_chat.id)

        return self._unwrap(response)

    def retrieve(
        self,
        chat_id: str,
        *,
        include: Optional[List[Literal["agent"]]] | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlaygroundChat:
        """Retrieve a specific playground chat by its ID.

        Parameters
        ----------
        chat_id : str
            ID of the playground chat to retrieve.
        include : list[Literal["agent"]] | None | Omit
            Related resources to embed in the response.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        PlaygroundChat
            The requested playground chat.

        Raises
        ------
        ValueError
            If ``chat_id`` is empty.
        """
        if not chat_id:
            raise ValueError(f"Expected a non-empty value for `chat_id` but received {chat_id!r}")

        response = self._get(
            f"/v2/playground-chats/{chat_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"include": include}, PlaygroundChatRetrieveParams),
            ),
            cast_to=APIResponseWithIncluded[PlaygroundChat, APIResponse[Agent]],
        )

        if include is not omit and include:
            response = embed_included_single(response, id_getter=lambda playground_chat: playground_chat.id)

        return self._unwrap(response)

    def delete(
        self,
        chat_id: str,
        *,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a single playground chat by its ID.

        Parameters
        ----------
        chat_id : str
            ID of the playground chat to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Raises
        ------
        ValueError
            If ``chat_id`` is empty.
        """
        if not chat_id:
            raise ValueError(f"Expected a non-empty value for `chat_id` but received {chat_id!r}")
        return self.bulk_delete(
            chat_ids=[chat_id],
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    def bulk_delete(
        self,
        *,
        chat_ids: SequenceNotStr[str],
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete multiple playground chats at once.

        Parameters
        ----------
        chat_ids : SequenceNotStr[str]
            IDs of the playground chats to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.
        """
        response = self._delete(
            "/v2/playground-chats",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"chat_ids": chat_ids}, PlaygroundChatBulkDeleteParams),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)


class AsyncPlaygroundChatsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPlaygroundChatsResourceWithRawResponse:
        return AsyncPlaygroundChatsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPlaygroundChatsResourceWithStreamingResponse:
        return AsyncPlaygroundChatsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        project_id: str,
        include: Optional[List[Literal["agent"]]] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        offset: Optional[int] | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[PlaygroundChat]:
        """List playground chats for a project with optional pagination.

        Parameters
        ----------
        project_id : str
            Project ID to list playground chats for.
        include : list[Literal["agent"]] | None | Omit
            Related resources to embed in the response (e.g. ``["agent"]``).
        limit : int | None | Omit
            Maximum number of playground chats to return.
        offset : int | None | Omit
            Number of results to skip for pagination.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list[PlaygroundChat]
            List of playground chats for the project.
        """
        response = await self._get(
            "/v2/playground-chats",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "project_id": project_id,
                        "include": include,
                        "limit": limit,
                        "offset": offset,
                    },
                    PlaygroundChatListParams,
                ),
            ),
            cast_to=APIResponseWithIncluded[List[PlaygroundChat], APIResponse[Agent]],
        )

        if include is not omit and include:
            response = embed_included_list(response, id_getter=lambda playground_chat: playground_chat.id)

        return self._unwrap(response)

    async def retrieve(
        self,
        chat_id: str,
        *,
        include: Optional[List[Literal["agent"]]] | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlaygroundChat:
        """Retrieve a specific playground chat by its ID.

        Parameters
        ----------
        chat_id : str
            ID of the playground chat to retrieve.
        include : list[Literal["agent"]] | None | Omit
            Related resources to embed in the response.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        PlaygroundChat
            The requested playground chat.

        Raises
        ------
        ValueError
            If ``chat_id`` is empty.
        """
        if not chat_id:
            raise ValueError(f"Expected a non-empty value for `chat_id` but received {chat_id!r}")

        response = await self._get(
            f"/v2/playground-chats/{chat_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"include": include}, PlaygroundChatRetrieveParams),
            ),
            cast_to=APIResponseWithIncluded[PlaygroundChat, APIResponse[Agent]],
        )

        if include is not omit and include:
            response = embed_included_single(response, id_getter=lambda playground_chat: playground_chat.id)

        return self._unwrap(response)

    async def delete(
        self,
        chat_id: str,
        *,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a single playground chat by its ID.

        Parameters
        ----------
        chat_id : str
            ID of the playground chat to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Raises
        ------
        ValueError
            If ``chat_id`` is empty.
        """
        if not chat_id:
            raise ValueError(f"Expected a non-empty value for `chat_id` but received {chat_id!r}")
        return await self.bulk_delete(
            chat_ids=[chat_id],
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    async def bulk_delete(
        self,
        *,
        chat_ids: SequenceNotStr[str],
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete multiple playground chats at once.

        Parameters
        ----------
        chat_ids : SequenceNotStr[str]
            IDs of the playground chats to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.
        """
        response = await self._delete(
            "/v2/playground-chats",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"chat_ids": chat_ids}, PlaygroundChatBulkDeleteParams),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)


class PlaygroundChatsResourceWithRawResponse:
    def __init__(self, playground_chats: PlaygroundChatsResource) -> None:
        self._playground_chats = playground_chats

        self.list = to_raw_response_wrapper(
            playground_chats.list,
        )
        self.retrieve = to_raw_response_wrapper(
            playground_chats.retrieve,
        )
        self.delete = to_raw_response_wrapper(
            playground_chats.delete,
        )
        self.bulk_delete = to_raw_response_wrapper(
            playground_chats.bulk_delete,
        )


class AsyncPlaygroundChatsResourceWithRawResponse:
    def __init__(self, playground_chats: AsyncPlaygroundChatsResource) -> None:
        self._playground_chats = playground_chats

        self.list = async_to_raw_response_wrapper(
            playground_chats.list,
        )
        self.retrieve = async_to_raw_response_wrapper(
            playground_chats.retrieve,
        )
        self.delete = async_to_raw_response_wrapper(
            playground_chats.delete,
        )
        self.bulk_delete = async_to_raw_response_wrapper(
            playground_chats.bulk_delete,
        )


class PlaygroundChatsResourceWithStreamingResponse:
    def __init__(self, playground_chats: PlaygroundChatsResource) -> None:
        self._playground_chats = playground_chats

        self.list = to_streamed_response_wrapper(
            playground_chats.list,
        )
        self.retrieve = to_streamed_response_wrapper(
            playground_chats.retrieve,
        )
        self.delete = to_streamed_response_wrapper(
            playground_chats.delete,
        )
        self.bulk_delete = to_streamed_response_wrapper(
            playground_chats.bulk_delete,
        )


class AsyncPlaygroundChatsResourceWithStreamingResponse:
    def __init__(self, playground_chats: AsyncPlaygroundChatsResource) -> None:
        self._playground_chats = playground_chats

        self.list = async_to_streamed_response_wrapper(
            playground_chats.list,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            playground_chats.retrieve,
        )
        self.delete = async_to_streamed_response_wrapper(
            playground_chats.delete,
        )
        self.bulk_delete = async_to_streamed_response_wrapper(
            playground_chats.bulk_delete,
        )



"""Test case domain types."""

from typing import List, Literal, Iterable, Optional, TypedDict
from datetime import datetime
from typing_extensions import Required

from pydantic import Field

from .chat import ChatMessage, ChatMessageParam, ChatMessageWithMetadataParam
from .user import UserReference
from .agent import AgentOutput
from .check import CheckConfig, TestCaseCheckConfigParam
from .._types import SequenceNotStr
from .._models import BaseModel

__all__ = [
    "TestCase",
    "TestCaseReference",
    "TestCaseComment",
    "BulkMoveTestCasesParams",
    "TestCaseCreateParams",
    "TestCaseUpdateParams",
    "TestCaseBulkDeleteParams",
    "TestCaseBulkUpdateParams",
    "CommentAddParams",
    "CommentEditParams",
]


# ---------------------------------------------------------------------------
# Models
# ---------------------------------------------------------------------------


class TestCaseComment(BaseModel):
    __test__ = False
    id: str
    content: str = Field(alias="comment")
    created_at: datetime
    updated_at: datetime
    user: UserReference


class TestCaseReference(BaseModel):
    __test__ = False
    id: str


class TestCase(BaseModel):
    __test__ = False
    id: str
    checks: List[CheckConfig]
    comments: List[TestCaseComment]
    created_at: datetime
    dataset_id: str
    demo_output: Optional[AgentOutput] = None
    messages: List[ChatMessage]
    tags: List[str]
    updated_at: datetime
    status: Literal["active", "draft"]


# ---------------------------------------------------------------------------
# Params
# ---------------------------------------------------------------------------


class TestCaseCreateParams(TypedDict, total=False):
    dataset_id: Required[str]
    messages: Required[Iterable[ChatMessageParam]]
    checks: Iterable[TestCaseCheckConfigParam]
    demo_output: Optional[ChatMessageWithMetadataParam]
    status: Optional[Literal["active", "draft"]]
    tags: SequenceNotStr[str]


class TestCaseUpdateParams(TypedDict, total=False):
    checks: Optional[Iterable[TestCaseCheckConfigParam]]
    dataset_id: Optional[str]
    demo_output: Optional[ChatMessageWithMetadataParam]
    messages: Optional[Iterable[ChatMessageParam]]
    tags: Optional[SequenceNotStr[str]]
    status: Optional[Literal["active", "draft"]]


class TestCaseBulkDeleteParams(TypedDict, total=False):
    test_case_ids: Required[SequenceNotStr[str]]


class TestCaseBulkUpdateParams(TypedDict, total=False):
    ids: Required[SequenceNotStr[str]]
    disabled_checks: Optional[SequenceNotStr[str]]
    enabled_checks: Optional[SequenceNotStr[str]]
    added_tags: Optional[SequenceNotStr[str]]
    removed_tags: Optional[SequenceNotStr[str]]
    status: Optional[Literal["active", "draft"]]


class BulkMoveTestCasesParams(TypedDict, total=False):
    chat_test_case_ids: Required[SequenceNotStr[str]]
    dataset_id: Required[str]
    duplicate: Optional[bool]


# ---------------------------------------------------------------------------
# Comment params
# ---------------------------------------------------------------------------


class CommentAddParams(TypedDict, total=False):
    comment: Required[str]


class CommentEditParams(TypedDict, total=False):
    comment: Required[str]
